@@ -136,12 +136,14 @@ sub list : Chained('base') Args(0) {
     my $limit = $c->req->params->{'limit'} || 10;
     my $sort  = $c->req->params->{'sort'}  || $info->{pk};
     (my $dir  = $c->req->params->{'dir'}   || 'ASC') =~ s/\s//g;
+    my $delayed_paging = 0; # in case we filter/sort on FK
 
     # sanity check the sort param
     $sort = $info->{pk} if $sort !~ m/^[\w ]+$/ or !exists $info->{cols}->{$sort};
 
-    # set up pager, if needed
-    my $search_opts = (($page =~ m/^\d+$/ and $limit =~ m/^\d+$/)
+    # set up pager, if needed (if user sorting by FK then delay paging)
+    my $search_opts = (
+        ($page =~ m/^\d+$/ and $limit =~ m/^\d+$/ and ! $info->{cols}->{$sort}->{is_fk})
         ? { 'page' => $page, 'rows' => $limit, } : {});
 
     # find filter fields in UI form
@@ -150,7 +152,12 @@ sub list : Chained('base') Args(0) {
         next unless $p =~ m/^search\.([\w ]+)/;
         my $col = $1;
         next unless exists $info->{cols}->{$col};
-        next if $info->{cols}->{$col}->{is_fk} or $info->{cols}->{$col}->{is_rr};
+        if ($info->{cols}->{$col}->{is_fk} or $info->{cols}->{$col}->{is_rr}) {
+            # sorry, have to kill the paging here because it breaks filtering on FK
+            $search_opts = {};
+            $delayed_paging = 1;
+            next;
+        }
 
         # for numberish types the case insensitive functions may not work
         if (exists $info->{cols}->{$col}->{extjs_xtype}
@@ -217,6 +224,22 @@ sub list : Chained('base') Args(0) {
         push @{$response->{rows}}, $data;
     }
 
+    # apply any filters on FK
+    foreach my $p (keys %{$c->req->params}) {
+        next unless $p =~ m/^search\.([\w ]+)/;
+        my $col = $1;
+        next unless exists $info->{cols}->{$col};
+        next unless ($info->{cols}->{$col}->{is_fk} or $info->{cols}->{$col}->{is_rr});
+
+        my $p_val = $c->req->params->{"search.$col"};
+        my $fk_match = ($p_val ? qr/\Q$p_val\E/i : qr/./);
+
+        # reduce to matching rows
+        $response->{rows} = [
+            grep { $_->{$col} =~ m/$fk_match/ } @{$response->{rows}}
+        ];
+    }
+
     # sort col which cannot be passed to the DB
     if ($info->{cols}->{$sort}->{is_fk} or $info->{cols}->{$sort}->{is_rr}) {
         @{$response->{rows}} = sort {
@@ -229,14 +252,26 @@ sub list : Chained('base') Args(0) {
     $response->{total} =
         eval {$rs->pager->total_entries} || scalar @{$response->{rows}};
 
+    # user sorted by FK so do the paging now (will be S-L-O-W)
+    if ($page =~ m/^\d+$/ and $limit =~ m/^\d+$/
+        and ($delayed_paging
+             or $info->{cols}->{$sort}->{is_fk}
+             or $info->{cols}->{$sort}->{is_rr})) {
+
+        my $pg = Data::Page->new;
+        $pg->total_entries(scalar @{$response->{rows}});
+        $pg->entries_per_page($limit);
+        $pg->current_page($page);
+        $response->{rows} = [ $pg->splice($response->{rows}) ];
+        $response->{total} = $pg->total_entries;
+    }
+
     # sneak in a 'top' row for applying the filters
     my %searchrow = ();
     foreach my $col (keys %{$info->{cols}}) {
         my $ci = $info->{cols}->{$col};
 
-        if (exists $ci->{is_fk}
-            or exists $ci->{is_rr}
-            or (exists $ci->{extjs_xtype} and $ci->{extjs_xtype} eq 'checkbox')) {
+        if (exists $ci->{extjs_xtype} and $ci->{extjs_xtype} eq 'checkbox') {
 
             $searchrow{$col} = '';
         }
@@ -343,6 +378,11 @@ sub _build_table_data {
             # FK val is an update, so set the value
             $data->{$col} = $params->{ 'combobox.' . $col } || undef
                 if exists $params->{ 'combobox.' . $col };
+
+            # rename col to real name, now we have data for it
+            # (custom relation accessor name)
+            $data->{ $ci->{masked_col} } = delete $data->{$col}
+                if defined $data->{$col} and exists $ci->{masked_col};
         }
         else {
         # not a foreign key, so just update the row data
@@ -406,6 +446,7 @@ sub _process_row_stack {
             my $ci = $info->{cols}->{$col};
             next unless exists $ci->{is_fk}
                 and exists $stashed_keys{$ci->{fk_model}};
+            $col = $ci->{masked_col} if exists $ci->{masked_col};
             $data->{$col} = $stashed_keys{$ci->{fk_model}};
         }
 
@@ -449,7 +490,7 @@ sub list_stringified : Chained('base') Args(0) {
     my $lf = $c->stash->{lf};
     my $response = $c->stash->{json_data} = {};
 
-    my $pg    = $c->req->params->{'page'}   || 1;
+    my $page  = $c->req->params->{'page'}   || 1;
     my $limit = $c->req->params->{'limit'}  || 5;
     my $query = $c->req->params->{'query'}  || '';
     my $fk    = $c->req->params->{'fkname'} || '';
@@ -474,13 +515,13 @@ sub list_stringified : Chained('base') Args(0) {
                 grep { _sfy($_) =~ m/$query/ } $rs->all;
     @data = sort { $a->{stringified} cmp $b->{stringified} } @data;
 
-    my $page = Data::Page->new;
-    $page->total_entries(scalar @data);
-    $page->entries_per_page($limit);
-    $page->current_page($pg);
+    my $pg = Data::Page->new;
+    $pg->total_entries(scalar @data);
+    $pg->entries_per_page($limit);
+    $pg->current_page($page);
 
-    $response->{rows} = [ $page->splice(\@data) ];
-    $response->{total} = $page->total_entries;
+    $response->{rows} = [ $pg->splice(\@data) ];
+    $response->{total} = $pg->total_entries;
 
     return $self;
 }
